/*
 * *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */
package org.nuiton.eugene;

import org.nuiton.eugene.models.Model;

import java.io.*;

/**
 * AbstractGenerator
 *
 * Created: 28 oct. 2009
 *
 * @param <M>
 * @author fdesbois
 * @version $Revision: 820 $
 *
 * Mise a jour: $Date: 2010-02-22 12:15:49 +0100 (lun., 22 févr. 2010) $
 * par : $Author: fdesbois $
 */
public abstract class AbstractGenerator<M extends Model> extends Template<M> {

    protected AbstractGenerator<M> parent = null;

    public AbstractGenerator() {
        super();
    }

    public AbstractGenerator(AbstractGenerator<M> parent) {
        super();
        setParent(parent);
    }

    public void setParent(AbstractGenerator<M> parent) {
        this.parent = parent;
        //setEncoding(parent.getEncoding());
    }

    @Override
    public boolean getOverwrite() {
        if (parent != null) {
            return parent.getOverwrite();
        }
        return super.getOverwrite();
    }

    @Override
    public String getEncoding() {
        if (parent != null) {
            return parent.getEncoding();
        }
        return super.getEncoding();
    }

    /**
     * Permet de recuperer la proprieté passé en argument
     *
     * @param name le nom de la propriete
     * @return la valeur de la propriété
     */
    @Override
    public String getProperty(String name) {
        if (parent != null) {
            return parent.getProperty(name);
        }
        return super.getProperty(name);
    }

    @Override
    public long getLastModifiedSource() {
        if (parent != null) {
            return parent.getLastModifiedSource();
        }
        return super.getLastModifiedSource();
    }

    /**
     * @param file fichier a tester
     * @return vrai si le fichier passé en parametre est plus recent que
     *         les sources sur generateur.
     */
    protected boolean isNewerThanSource(File file) {
        if (log.isDebugEnabled()) {
            log.debug("source date: " + getLastModifiedSource()
                    + " file date: " + file.lastModified() + "(" + file + ")");
        }
        return file.lastModified() > getLastModifiedSource();
    }

    protected File getDestinationFile(File destDir, String filename) {
        return new File(destDir, filename);
    }

    /**
     * Test if given package is allowed for generation.
     *
     * An element can be generated if his package is in the {@link #generatedPackages} list
     * or if {@link #generatedPackages} is null or empty.
     *
     * @param packageName package name to test
     * @return generation allowed
     */
    protected boolean canGeneratePackage(String packageName) {
        boolean canGenerate = true;
        if (parent != null) {
            return parent.canGeneratePackage(packageName);
        }
        // if not generation restriction, generate everything
        if (getGeneratedPackages() != null && !getGeneratedPackages().isEmpty()) {
            canGenerate = false;

            // on doit donc tester que le package courant
            // soit est egal a un des element de la liste
            // soit commence par ca
            for (String generatedPackage : getGeneratedPackages()) {

                // cas egalité parfaites
                if (packageName != null && packageName.equals(generatedPackage)) {
                    canGenerate = true;
                }

                // cas commence par (on ajout un point à la fin)
                String localGeneratedPackage = generatedPackage;
                if (!localGeneratedPackage.endsWith(".")) {
                    localGeneratedPackage += ".";
                }
                if (packageName != null && packageName.startsWith(localGeneratedPackage)) {
                    canGenerate = true;
                }

            }
        }

        return canGenerate;
    }

    protected void write(File outputFile, MonitorWriter out) {
        File write0utputFile = outputFile;
        if (out.isModified()) {
            try {
                write0utputFile = outputFile.getCanonicalFile();

                if (!write0utputFile.getParentFile().exists()) {
                    boolean b = write0utputFile.getParentFile().mkdirs();
                    if (!b) {
                        throw new IllegalStateException("could not create directory " + write0utputFile.getParentFile());
                    }
                }

                Writer output;

                String encoding = getEncoding();
                if (encoding != null) {
                    if (log.isDebugEnabled()) {
                        log.debug("Force encoding to " + encoding + " : " + this);
                    }
                    output = new OutputStreamWriter(new FileOutputStream(
                            write0utputFile), encoding);
                } else {
                    output = new OutputStreamWriter(new FileOutputStream(
                            write0utputFile));
                }

                output.write(out.getBuffer().toString());
                output.close();
            } catch (IOException eee) {
                if (log.isWarnEnabled()) {
                    log.warn("Unable to write file : " + write0utputFile.getAbsolutePath(), eee);
                }
                throw new RuntimeException(eee);
            }
        }
    }
}
