/* *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */

package org.nuiton.eugene;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.models.Model;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Generator. TODO javadoc
 *
 * Created: 5 août 2004
 *
 * @param <M> Model associated to the generator (input model)
 * 
 * @author Cedric Pineau <pineau@codelutin.com>
 * @version $Revision: 820 $
 *
 * Mise a jour: $Date: 2010-02-22 12:15:49 +0100 (lun., 22 févr. 2010) $
 * par : $Author: fdesbois $
 */
public abstract class Template<M extends Model> {

    protected static Log log = LogFactory.getLog(Template.class);

    public static final String PROP_DEFAULT_PACKAGE = "defaultPackage";
    public static final String PROP_OVERWRITE = "overwrite";
    public static final String PROP_ENCODING = "encoding";
    public static final String PROP_LAST_MODIFIED_SOURCE = "lastModifiedSource";
    public static final String PROP_GENERATED_PACKAGES = "generatedPackages";
    public static final String PROP_EXCLUDE_TEMPLATES = "excludeTemplates";

    protected Properties properties = new Properties();
    //protected boolean overwrite = true;

    protected List<String> excludeTemplates;

    /** 
     * List of package to allow generation.
     * 
     * If {@code null} or empty, generate all packages.
     */
    protected List<String> generatedPackages;
    
//    protected String encoding;

    /** date de derniere modification de la source la plus recente */
    //protected long lastModifiedSource = 0;

    /** Model */
    protected M model;

    /**
     * Old method from EUGene 1.0.1 used to generate files. Now, you have to use a Reader instead of
     * directly use a Template.
     *
     * @param file list of tiles
     * @param destDir destination folder
     * @deprecated since 2.0.0
     */
    @Deprecated
    public abstract void generate(File[] file, File destDir);

    public abstract void applyTemplate(M model, File destDir) throws IOException;
    

    /**
     * @param v override value
     * @deprecated since 2.0.0, use a Properties object to add the {@link Template#PROP_OVERWRITE} value
     * @see Template#setProperties(java.util.Properties)
     */
    @Deprecated
    public void setOverwrite(boolean v) {
        //overwrite = v;
        properties.setProperty(PROP_OVERWRITE, String.valueOf(v));
    }

    public boolean getOverwrite() {
        return Boolean.parseBoolean(properties.getProperty(PROP_OVERWRITE, "true"));
    }

    public String getEncoding() {
        return properties.getProperty(PROP_ENCODING, "");
    }

    /**
     * @param encoding encoding value
     * @deprecated since 2.0.0, use a Properties object to add the {@link Template#PROP_ENCODING} value
     * @see Template#setProperties(java.util.Properties)
     */
    @Deprecated
    public void setEncoding(String encoding) {
        //this.encoding = encoding;
        properties.setProperty(PROP_ENCODING, encoding);
    }

    public void setProperties(Properties p) {
        this.properties = p;
    }

    /**
     * Permet de recuperer la proprieté passé en argument
     *
     * @param name le nom de la propriete
     * @return la valeur de la propriété
     */
    public String getProperty(String name) {
        return properties.getProperty(name);
    }
    
    /** 
     * Set list of package to allow generation.
     * 
     * If {@code null} or empty, generate all packages.
     * 
     * @param generatedPackages list of package to generate
     * @deprecated 
     */
    @Deprecated
    public void setGeneratedPackages(List<String> generatedPackages) {
        this.generatedPackages = generatedPackages;
    }

    protected List<String> getGeneratedPackages() {
        if (generatedPackages == null) {
            generatedPackages = new ArrayList<String>();
            String genPackages = getProperty(PROP_GENERATED_PACKAGES);
            if (!StringUtils.isEmpty(genPackages)) {
                //TC-20091125 avoid manual array copy
                generatedPackages.addAll(Arrays.asList(genPackages.split(",")));
//                for (String s : genPackages.split(",")) {
//                    generatedPackages.add(s);
//                }
            }
        }     
        return this.generatedPackages;
    }

    /**
     * @param lastModifiedSource lastModifiedSource value
     * @deprecated since 2.0.0, use a Properties object to add the {@link Template#PROP_LAST_MODIFIED_SOURCE} value
     * @see Template#setProperties(java.util.Properties)
     */
    @Deprecated
    public void setLastModifiedSource(long lastModifiedSource) {
        properties.setProperty(PROP_LAST_MODIFIED_SOURCE, String.valueOf(lastModifiedSource));
    }

    public long getLastModifiedSource() {
        return Long.parseLong(properties.getProperty(PROP_LAST_MODIFIED_SOURCE, "0"));
    }
    
    public List<String> getExcludeTemplates() {
        if (excludeTemplates == null) {
            excludeTemplates = new ArrayList<String>();
            String excludes = getProperty(PROP_EXCLUDE_TEMPLATES);
            if (!StringUtils.isEmpty(excludes)) {
                //TC-20091125 avoid manual array copy
                excludeTemplates.addAll(Arrays.asList(excludes.split(",")));
//                for (String s : excludes.split(",")) {
//                    excludeTemplates.add(s);
//                }
            }
        }
        return excludeTemplates;
    }

    @Deprecated
    public void setExcludeTemplates(List<String> excludeTemplates) {
        this.excludeTemplates = excludeTemplates;
    }

    public M getModel() {
        return this.model;
    }

}
