package org.nuiton.eugene.java;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.models.object.ObjectModelClassifier;
import org.nuiton.eugene.models.object.ObjectModelElement;

import java.util.HashMap;
import java.util.Map;

/**
 * Created: 17 déc. 2009
 *
 * @author Tony Chemit <chemit@codelutin.com> Copyright Code Lutin
 * @version $Revision: 797 $
 *          <p/>
 *          Mise a jour: $Date: 2010-01-17 21:43:08 +0100 (dim., 17 janv. 2010) $ par :
 *          $Author: tchemit $
 */
public class AnnotationsManagerExtension {

    private static final Log log = LogFactory.getLog(AnnotationsManagerExtension.class);

    /**
     * Extension static used to identify AnnotationsManagerExtension in ObjectModel
     */
    public static final String OBJECTMODEL_EXTENSION = "annotations";

    /**
     * Map of AnotationsManager with key equals to the classifier qualified name associated to
     * the AnotationsManager
     */
    protected Map<String, AnnotationsManager> managers;

    /**
     * Get the registred annotations for the given {@code element} in the given {@code classifier}.
     *
     * <b>Note:</b> The method always returns a {@code none null} value, but
     * an empty array when no annotation when no annotation found for the element. 
     * 
     * @param classifier the classifier where is the element
     * @param element the element on which searching annotations
     * @return the array of annotation registred or an empty array if none.
     */
    public String[] getAnnotations(ObjectModelClassifier classifier, ObjectModelElement element) {
        AnnotationsManager annotationsManager = getManager(classifier);
        String[] result = null;
        if (annotationsManager != null) {
            result = annotationsManager.getAnnotations(element);
        }
        return result == null ? new String[]{} : result;
    }

    /**
     * Get the AnotationsManager associated to the classifier. If not exist, it will be created.
     *
     * @param classifier reference for the AnotationsManager
     * @return the annotationsManager associated to the classifier (never null)
     */
    public AnnotationsManager getManager(ObjectModelClassifier classifier) {
        Map<String, AnnotationsManager> managers = getManagers();
        String fqn = classifier.getQualifiedName();
        AnnotationsManager manager = managers.get(fqn);
        if (manager == null) {
            manager = new AnnotationsManager();
            managers.put(fqn, manager);
            if (log.isDebugEnabled()) {
                log.debug("Add new annotationsManager for : " + fqn);
            }
        }
        return manager;
    }

    protected Map<String, AnnotationsManager> getManagers() {
        if (managers == null) {
            managers = new HashMap<String, AnnotationsManager>();
        }
        return managers;
    }
}
